@@ -537,6 +537,14 @@ plotSurvd <- function(ds,ids=NULL,add=FALSE,se=FALSE,cols=NULL,ltys=NULL,...)
 ##'    P(T >t | x) =  \frac{1}{1 + G(t) exp( x \beta) }
 ##' }
 ##' 
+##' This is thus also the cumulative odds model, since 
+##' \deqn{
+##'    P(T \leq t | x) =  \frac{G(t) \exp(x \beta) }{1 + G(t) exp( x \beta) }
+##' }
+##' 
+##' The baseline \eqn{G(t)} is written as \eqn{cumsum(exp(\alpha))} and this is not the standard
+##' parametrization that takes log of \eqn{G(t)} as the parameters.
+##' 
 ##' Input are intervals given by ]t_l,t_r] where t_r can be infinity for right-censored intervals 
 ##' When truly discrete ]0,1] will be an observation at 1, and  ]j,j+1] will be an observation at j+1
 ##' 
@@ -563,31 +571,9 @@ plotSurvd <- function(ds,ids=NULL,add=FALSE,se=FALSE,cols=NULL,ltys=NULL,...)
 ##' out <- interval.logitsurv.discrete(Interval(entry,time2)~X1+X2+X3+X4,ttpd)
 ##' summary(out)
 ##' 
-##' n <- 100
-##' Z <- matrix(rbinom(n*4,1,0.5),n,4)
-##' outsim <- simlogitSurvd(out$coef,Z)
-##' outsim <- transform(outsim,left=time,right=time+1)
-##' outsim <- dtransform(outsim,right=Inf,status==0)
-##' 
-##' outss <- interval.logitsurv.discrete(Interval(left,right)~+X1+X2+X3+X4,outsim)
-##' 
-##' Z <- matrix(0,5,4)
-##' Z[2:5,1:4] <- diag(4)
 ##' pred <- predictlogitSurvd(out,se=FALSE)
 ##' plotSurvd(pred)
 ##' 
-##' ## simulations 
-##' n <- 100
-##' Z <- matrix(rbinom(n*4,1,0.5),n,4)
-##' outsim <- simlogitSurvd(out$coef,Z)
-##' ###
-##' outsim <- transform(outsim,left=time,right=time+1)
-##' outsim <- dtransform(outsim,right=Inf,status==0)
-##' 
-##' out$coef
-##' outss <- interval.logitsurv.discrete(Interval(left,right)~+X1+X2+X3+X4,outsim)
-##' summary(outss)
-##' 
 ##' @aliases Interval dInterval simlogitSurvd predictlogitSurvd
 ##' @export
 interval.logitsurv.discrete <- function (formula,data,beta=NULL,no.opt=FALSE,method="NR",
@@ -615,6 +601,8 @@ interval.logitsurv.discrete <- function (formula,data,beta=NULL,no.opt=FALSE,met
 	if (max(entrytime)==0) left <- 0
     }
 
+  if (any(entrytime==time2)) stop("left==right not possible for discrete data")
+
   id <- strata <- NULL
   if (!is.null(attributes(Terms)$specials$cluster)) {
     ts <- survival::untangle.specials(Terms, "cluster")
@@ -636,6 +624,16 @@ interval.logitsurv.discrete <- function (formula,data,beta=NULL,no.opt=FALSE,met
   if (ncol(X)>0) X.names <- colnames(X) else X.names <- NULL
 ###  if (ncol(X)==0) X <- NULL 
 
+  if (!is.null(id)) {
+	  ids <- unique(id)
+	  nid <- length(ids)
+      if (is.numeric(id)) id <-  fast.approx(ids,id)-1 else  {
+      id <- as.integer(factor(id,labels=seq(nid)))-1
+     }
+   } else { id <- as.integer(seq_along(time2))-1; nid <- length(time2) }
+   ## orginal id coding into integers 
+   id.orig <- id+1; 
+
   ## times 1 -> mutimes , 0 til start
   utimes <- sort(unique(c(time2,entrytime)))
 ### time2 <- fast.approx(utimes,time2)-1
@@ -669,11 +667,16 @@ interval.logitsurv.discrete <- function (formula,data,beta=NULL,no.opt=FALSE,met
   ## weights/offets will follow id 
   if (is.null(weights))  weights <- rep(1,n); #  else wiid <- weights
   if (is.null(offsets))  offsets <- rep(0,n); # else offsets <- offsets
-  if (is.null(beta)) beta <- rep(0,ncol(X)+mutimes)
   expit  <- function(z) 1/(1+exp(-z)) ## expit
   logit  <- function(p) log(p/(1-p))  ## logit
 
-  beta[1:mutimes] <- (1:mutimes)*exp(logit( (sum(time2<Inf)/(nrow(X)*mutimes))))
+  if (is.null(beta)) {
+     beta <- rep(0,ncol(X)+mutimes)
+     Set <- 1-cumsum(table(time2))/n
+     dHt <- log(diff(c(0,(1/Set-1))))
+     beta[1:mutimes] <- dHt[1:mutimes]
+  }
+
 
 obj <- function(pp,all=FALSE)
 { # {{{
@@ -782,9 +785,10 @@ hessian <- D2log
   if (all) {
       ihess <- solve(hessian)
       beta.iid <- Dlogliid %*% ihess ## %*% t(Dlogl) 
+      beta.iid <- apply(beta.iid,2,sumstrata,id,nid)
       robvar <- crossprod(beta.iid)
       val <- list(par=pp,ploglik=ploglik,gradient=gradient,hessian=hessian,ihessian=ihess,
-		  iid=beta.iid,robvar=robvar,var=-ihess,
+		  iid=beta.iid,robvar=robvar,var=-ihess,id=id,
 		  se=diag(-ihess)^.5,se.robust=diag(robvar)^.5)
       return(val)
   }  
